<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Articulo;
use App\Models\Categoria;
use App\Models\Proveedor;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class ArticuloController extends Controller
{
    public function index()
    {

        $articulo = DB::table('articulos as art')
            ->join('categoria as cat', 'art.categoria_id', '=', 'cat.id')
            ->select(
                'cat.nombre as nomCat',
                'art.nombre',
                'art.cod_barras',
                'art.descripcion',
                'art.p_venta',
                'art.stock',
                'art.id',
                'art.tipo',
                'art.min_stock',
            )
            ->where('art.deleted_at', null)
            ->get();

        return view('admin.articulos.index', ['articulos' => $articulo, 'categoria' => Categoria::all()]);
    }

    public function create()
    {
        return view('admin.articulos.create', ['categoria' => Categoria::all(), 'proveedor' => Proveedor::all()]);
    }

    public function store(Request $request)
    {


        // if (Articulo::where('cod_barras', $request->cod_barras)->first()) {
        //     return back()->with(['info' => "Articulo con codigo de barras ya registrado", 'color' => 'info']);
        // }


        $articulo = new Articulo();
        $articulo->nombre = $request->nombre;
        $articulo->categoria_id = $request->categoria_id;
        $articulo->proveedor_id = $request->proveedor_id;
        $code = '';
        if (!$request->cod_barras) {
            $code = date("mYdihs");
            $articulo->cod_barras = $code;
        } else {

            $articulo->cod_barras = $request->cod_barras;
        }

        $articulo->descripcion = $request->descripcion;
        $articulo->descripcion_interna = $request->descripcion_interna;
        $articulo->p_venta = $request->p_venta;
        $articulo->p_costo = $request->p_costo;
        $articulo->stock = $request->stock;
        $articulo->img = $request->img;
        $articulo->tipo = $request->tipo;

        $articulo->min_stock = $request->min_stock;
        $articulo->p_descuento = $request->p_descuento;
        $articulo->url_articulo = $request->url_articulo;
        $articulo->fecha_ven = $request->fecha_ven;
        $articulo->save();

        return back()->with(['info' => "Articulo $articulo->nombre registrado con exito", 'color' => 'success']);
    }

    public function show($id)
    {
        $articulo = DB::table('articulos as art')
            ->join('categoria as cat', 'art.categoria_id', '=', 'cat.id')
            ->join('proveedores as prove', 'art.proveedor_id', '=', 'prove.id')
            ->select(
                'cat.nombre as nomCat',
                'cat.tipo',
                'art.nombre',
                'art.cod_barras',
                'art.descripcion',
                'art.p_venta',
                'art.p_costo',
                'art.stock',
                'art.img',
                'art.id',
                'art.fecha_promo',
                'art.created_at',
                'art.descripcion_interna',
                'art.min_stock',
                'art.p_descuento',
                'art.categoria_id',
                'art.url_articulo',
                'art.fecha_ven',
                'art.tipo',
                'prove.nombre as provName',
                'prove.empresa',
                'prove.direccion',
                'prove.telefono1',
                'prove.telefono2',
                'prove.id as proveId',
                'prove.articulos'
            )
            ->where('art.id', $id)
            ->first();


        return view('admin.articulos.show', ['articulo' => $articulo, 'categoria' => Categoria::all(), 'proveedor' => Proveedor::all()]);
    }

    public function update(Request $request, $id)
    {
        $this->actualizar($request, $id);
        return back()->with(['info' => "Articulo actualizado con exito", 'color' => 'warning']);
    }

    public function destroy($id)
    {
        $articulo = Articulo::find($id);
        $articulo->delete();
        return back()->with(['info' => "Articulo $articulo->nombre  $articulo->descripcion Eliminado con exito", 'color' => 'danger']);
    }

    // retorna la vista pra el control de stock
    public function controlStock()
    {
        $articulo = Articulo::all();
        return view('admin.articulos.contro_stock', ['articulos' => $articulo]);
    }

    public function actualizar(Request $request, $id)
    {
        return  Articulo::find($id)->update($request->all());
    }

    // devuelve la lista de reportes para articulos
    public function reportes()
    {
        return view('admin.articulos.reportes');
    }

    // devuelve un listado de todos los articulos
    public function reportAll()
    {
        $articulos = DB::table('articulos as art')
            ->join('categoria as cat', 'art.categoria_id', '=', 'cat.id')
            ->select('cat.nombre as nomCat', 'cat.tipo', 'art.nombre', 'art.cod_barras', 'art.descripcion', 'art.p_venta', 'art.p_costo', 'art.stock', 'art.img', 'art.id')
            ->where('art.deleted_at', null)
            ->get();
        $total = 0;

        foreach ($articulos as $art) {
            $total++;
        }

        return view('admin.reports.articulos.all', ['articulos' => $articulos, 'total' => $total]);
    }

    // devuelve un listado de todos los articulos con stock < 5
    public function reportAllmin()
    {
        $articulos = Articulo::all();
        $total = 0;


        return view('admin.reports.articulos.allmin', ['articulos' => $articulos, 'total' => $total]);
    }

    public function stockControl()
    {
        return view('admin.reports.articulos.stock', ['articulos' => Articulo::all(), 'total' => Articulo::count()]);
    }

    public function articulosvendidos(Request $data)
    {
        $vendidios = DB::table('venta_articulos AS ven')
            ->join('clientes AS cli', 'ven.cliente_id', '=', 'cli.id')
            ->join('articulos AS art', 'ven.articulo_id', '=', 'art.id')
            ->join('facturas AS fac', 'ven.factura_id', '=', 'fac.id')
            ->select(
                'ven.id',
                'ven.factura_id',
                'ven.articulo_id',
                'ven.cantidad',
                'ven.created_at as fechaVenta',
                'art.nombre as nomArt',
                'art.cod_barras',
                'art.descripcion'

            )
            ->whereMonth('ven.created_at', $data->mes)
            ->whereYear('ven.created_at', $data->ano)
            ->where('art.deleted_at', null)

            ->get();


        return view('admin.reports.articulos.vendidos', ['articulos' => $vendidios]);
    }

    // imprime el codigo de barras de un articulos 
    public function dataPrint($id)
    {
        $articulo = Articulo::find($id);
        return view('admin.reports.articulos.print-data', ['articulo' => $articulo]);
    }

    // RETORNA LOS ARTICULOS POR TIPO
    public function reportArticuloToTipo(Request $request)
    {
        $articulos = DB::table('articulos as art')
            ->join('categoria as cat', 'art.categoria_id', '=', 'cat.id')
            ->select('cat.nombre as nomCat', 'cat.tipo', 'art.nombre', 'art.cod_barras', 'art.descripcion', 'art.p_venta', 'art.p_costo', 'art.stock', 'art.img', 'art.id')
            ->where('art.deleted_at', null)
            ->where('art.tipo', $request->tipo)
            ->get();
        $total = 0;

        foreach ($articulos as $art) {
            $total++;
        }

        return view('admin.reports.articulos.all', ['articulos' => $articulos, 'total' => $total]);
    }
}
