<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\AbonoCliente;
use App\Models\Caja;
use App\Models\Cliente;
use App\Models\DeudaCliente;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class ClienteController extends Controller
{
    // devuelve el lista dode los clientes
    public function index()
    {
        return view('admin.clientes.index', ['clientes' => Cliente::all()]);
    }

    // muetra el perfil de un cliente
    public function show($id)
    {

        $deudas =  DeudaCliente::where('cliente_id', $id)
            ->where('estado', 'DEUDA')
            ->get();

        $pagos = DeudaCliente::where('cliente_id', $id)
            ->where('estado', 'PAGADO')
            ->get();
        $abonos =  AbonoCliente::where('cliente_id', $id)->get();

        return view('admin.clientes.show', ['cliente' => Cliente::find($id), 'deudas' => $deudas, 'pagos' => $pagos, 'abonos' => $abonos]);
    }

    // acctualiza un cliente
    public function update(Request $data, $id)
    {
        $cliente = Cliente::find($id);

        $cliente->nombre = $data->nombre;
        $cliente->nit = $data->nit;
        $cliente->direccion = $data->direccion;
        $cliente->telefono1 = $data->telefono1;
        $cliente->telefono2 = $data->telefono2;
        $cliente->save();

        return back()->with(['info' => "Cliente $cliente->nombre Actualizado Con Exito", 'color' => 'warning']);
    }

    // elimiena un cliente
    public function delete($id)
    {
        $cliente = Cliente::find($id);

        $cliente->delete();
        return back()->with(['info' => "Cliente $cliente->nombre Elimindao Con Exito", 'color' => 'danger']);
    }

    // vista para crear cliene
    public function create()
    {
        return view('admin.clientes.create');
    }

    // gaurada un cliente
    public function store(Request $data)
    {
        $cliente = new Cliente();
        $cliente->nombre = $data->nombre;
        $cliente->nit = $data->nit;
        $cliente->direccion = $data->direccion;
        $cliente->telefono1 = $data->telefono1;
        $cliente->telefono2 = $data->telefono2;
        $cliente->save();
        return back()->with(['info' => "Cliente $cliente->nombre Registrado Con Exito, Su Codigo es:  - $cliente->id -", 'color' => 'success']);
    }

    // devuelve la lista de los deudores para acceder a su perfil
    public function deudores()
    {
        $deudores = $this->deudoresFunction();

        return view('admin.clientes.deudores', ['deudores' => $deudores]);
    }

    // devuelve la vista para imprimir las deudas del cliente
    public function reporteDeudore($id)
    {
        $deuda = DB::table('venta_articulos AS ven')
            ->join('clientes AS cli', 'ven.cliente_id', '=', 'cli.id')
            ->join('articulos AS art', 'ven.articulo_id', '=', 'art.id')
            ->select(
                'ven.id',
                'ven.factura_id',
                'ven.cantidad',
                'ven.cliente_id',
                'ven.total',
                'ven.credito',
                'ven.created_at as fechaVenta',
                'cli.nombre',
                'cli.direccion',
                'cli.telefono1',
                'cli.telefono2',
                'art.nombre as nomArt',
                'art.cod_barras',
                'art.descripcion'
            )
            ->where('cliente_id', $id)
            ->where('ven.credito', 1)
            ->get();

        $totalDeuda = 0;
        $nombre = '';
        foreach ($deuda as $deu) {
            $totalDeuda =  $deu->total + $totalDeuda;
            $nombre = $deu->nombre;
        }



        return view('admin.reports.clientes.deudascliente', ['deuda' => $deuda, 'totalDeuda' => $totalDeuda, 'cliente' => $nombre]);
    }

    // devuelve el listado disponible para clientes 
    public function reportes()
    {
        return view('admin.clientes.reportes');
    }


    // devuenve la vista de toos los deuduoires
    public function reportesAllDeudores()
    {
        return view('admin.reports.clientes.deudores', ['deudores' => $this->deudoresFunction()]);
    }


    // reporte gneral de clietes
    public function reportesAllClientes()
    {
        $clientes = Cliente::all();
        $total = 0;

        foreach ($clientes as $cli) {
            $total++;
        }



        return view('admin.reports.clientes.clientes', ['clientes' => $clientes, 'total' => $total]);
    }

    // para reciclado0 de codigo = deurodores alll
    public function deudoresFunction()
    {
        // devuelve la consulta de creditos
        $deuda = DB::table('venta_articulos AS ven')
            ->join('clientes AS cli', 'ven.cliente_id', '=', 'cli.id')
            ->join('articulos AS art', 'ven.articulo_id', '=', 'art.id')
            ->select(
                'ven.factura_id',
                'ven.cantidad',
                'ven.total',
                'ven.cliente_id',
                'ven.created_at as fechaVenta',
                'cli.nombre',
                'cli.direccion',
                'cli.telefono1',
                'cli.telefono2',
                'art.nombre as nomArt',
                'art.cod_barras',
                'art.descripcion',
                'art.img'
            )
            ->where('ven.credito', 1)
            ->groupBy('cli.id')
            ->get();

        return $deuda;
    }


    // guarda la deuda de un cliente
    public function saveDeuda(Request $request)
    {
        DeudaCliente::create($request->all());
        $cli = Cliente::findOrFail($request->cliente_id);


        $caja = new Caja();
        $caja->valor = $request->total;
        $caja->descripcion = "CREDITO DE $cli->nombre por: $request->descripcion";
        $caja->operacion = 'd';
        $caja->fecha = date('Y-m-d');
        $caja->save();

        return back()->with(['info' => "CREDITO ASIGNADO", 'color' => 'success']);
    }

    // cambia el estado de una deuda a cambiado
    public function pagarDeuda($id)
    {
        $deu = DeudaCliente::findOrFail($id);
        $deu->estado = 'PAGADO';
        $deu->save();

        return back()->with(['info' => "deuda actualizada", 'color' => 'success']);
    }


    // asigna el abono de una deuda
    public function abonosNew(Request $request)
    {

        AbonoCliente::create($request->all());
        $cli = Cliente::findOrFail($request->cliente_id);

        $caja = new Caja();
        $caja->valor = $request->total;
        $caja->descripcion = "ABONO  $cli->nombre por: $request->descripcion";
        $caja->operacion = 'e';
        $caja->fecha = date('Y-m-d');
        $caja->save();


        return back()->with(['info' => "ABONO REALIZADO CON EXITO", 'color' => 'success']);
    }
}
