<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Articulo;
use App\Models\Compra;
use App\Models\CompraProveedores;
use App\Models\Proveedor;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class CompraController extends Controller
{
    // retorna la vista para comprar articulos
    public function index(Request $request)
    {
        $prove = Proveedor::all();

        $vis = false;
        $articulos = null;

        if ($request->proveeedor) {
            $vis = true;
            $articulos = DB::table('articulos as art')
                ->join('categoria as cat', 'art.categoria_id', '=', 'cat.id')
                ->select('cat.nombre as nomCat', 'cat.tipo', 'art.nombre', 'art.cod_barras', 'art.descripcion', 'art.p_venta', 'art.p_costo', 'art.stock', 'art.img', 'art.id')
                ->where('art.deleted_at', null)
                ->where('proveedor_id', $request->proveeedor)
                ->get();
        }


        return view('admin.articulos.compras', ['articulos' => $articulos, 'prove' => $prove, 'vis' => $vis]);
    }

    public function store(Request $request)
    {
        $articulo = $this->guardar($request);
        return back()->with(['info' => "Factura $articulo, registrada con Exito", 'color' => 'success']);
    }

    // retorna la vista de historial
    public function shows()
    {

        $historias = DB::table('compra_articulos as ca')
            ->join('articulos as art', 'ca.articulo_id', '=', 'art.id')
            ->join('users as use', 'ca.user_id', '=', 'use.id')
            ->select('art.nombre', 'art.descripcion', 'art.img', 'ca.id as idCom', 'ca.factura', 'ca.cantidad', 'ca.fecha', 'art.id', 'ca.user_id', 'use.name')
            ->orderBy('ca.id', 'DESC')
            ->get();

        return view('admin.articulos.historialCompras', ['articlos' => $historias]);
    }

    // guarda la compra de un articulo
    public function guardar(Request $request)
    {
        for ($i = 0; $i < count($request->id_bolsas); $i++) {

            $proveedor = new CompraProveedores();
            $fecha_actual = date("d-m-Y");
            $compra = new Compra();
            $compra->articulo_id = $request->id_bolsas[$i];
            $compra->user_id = Auth::user()->id;
            $compra->cantidad = $request->cantidad_bolsa[$i];
            $compra->factura = $request->descripcion;
            $compra->fecha = date('Y-m-d');
            $compra->fecha_vencimiento =   date("Y-m-d", strtotime($fecha_actual . "+ 1 year"));

            $articulo = Articulo::where('id', $request->id_bolsas[$i])->first();
            $articulo->stock = $articulo->stock + (float) $request->cantidad_bolsa[$i];


            $proveedor->articulo_id = (int) $request->id_bolsas[$i];
            $proveedor->proveedor_id = $articulo->proveedor_id;
            $proveedor->cantidad = $request->cantidad_bolsa[$i];
            $proveedor->factura = $request->descripcion;
            $proveedor->fecha_de_compra = date('Y-m-d');
            
            // dd($proveedor);

            $proveedor->save();
            $compra->save();
            $articulo->save();
        }
        return $request->descripcion;
    }

    public function delete($id, $idArt)
    {
        $compra = Compra::find($id);

        $articulo = Articulo::find($idArt);

        $articulo->stock = $articulo->stock - $compra->cantidad;

        $articulo->save();
        $compra->delete();

        return back()->with(['info' => "Compra $articulo->nombre $articulo->descripcion Eliminda con exito", 'color' => 'danger']);
    }

    public function report()
    {
        $historias = DB::table('compra_articulos as ca')
            ->join('articulos as art', 'ca.articulo_id', '=', 'art.id')
            ->join('users as user', 'ca.user_id', '=', 'user.id')
            ->select(
                'art.nombre',
                'art.descripcion',
                'art.img',
                'ca.id as idCom',
                'ca.factura',
                'ca.cantidad',
                'ca.fecha',
                'art.id',
                'user.name as userName',
            )
            ->orderBy('ca.id', 'DESC')
            ->get();
        return view('admin.reports.articulos.comprasHistorial', ['historial' => $historias]);
    }
}
