<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Articulo;
use App\Models\Cliente;
use App\Models\Factura;
use App\Models\Ventas;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Response;

class VentasController extends Controller
{
    public function index()
    {
        $articulos = DB::table('articulos as art')
            ->join('categoria as cat', 'art.categoria_id', '=', 'cat.id')
            ->select('art.id', 'art.nombre', 'art.stock', 'art.p_venta', 'art.cod_barras as code', 'art.descripcion', 'cat.nombre as catNom', 'art.descripcion_interna', 'art.img')
            ->where('art.stock', '>', 1000)
            ->where('art.deleted_at', null)
            ->get();
        $cliente = Cliente::all();
        return view('admin.ventas.index', ['articulos' => $articulos, 'clientes' => $cliente]);
    }

    // crea una venta
    public function store(Request $data)
    {
        $vuelta =  count($data->articulo_id);
        $factura = new Factura();
        $factura->descripcion = $data->descripcion;
        $factura->total = $data->totalventa;
        $factura->factura = 'NO';

        if ($data->nombre && $data->nit) {
            $factura->factura = 'SI';
            $factura->nombre = $data->nombre;
            $factura->nit = $data->nit;
            $factura->direccion = $data->direccion;
        } else {
            $factura->factura = 'NO';
            $factura->nombre = 'CONSUMIDOR FINAL';
            $factura->nit = 'CF';
            $factura->direccion = 'CIUDAD';
        }
        $factura->save();


        $cliente = $data->cliente;

        $credito = false;
        if ($data->credito) {
            $credito = true;
        }
        if ($data->credito == null) {
            $credito = false;
        }

        if (!$data->cliente) {
            $cliente = 14;
        }

        for ($i = 0; $i < $vuelta; $i++) {
            $venta = new Ventas();
            $venta->articulo_id = $data->articulo_id[$i];
            $venta->cliente_id = $cliente;
            $venta->factura_id = $factura->id;
            $venta->credito = $credito;
            $venta->cantidad = $data->cantidad[$i];
            $venta->descuento = $data->descuento[$i];
            $venta->total = $data->subtotal[$i];
            $venta->user_id = Auth::user()->id;
            $venta->save();


            $art = Articulo::find($data->articulo_id[$i]);
            $art->stock -= $data->cantidad[$i];
            $art->save();

        }

        return back()->with(['info' => 'Venta Registrada con Exito', 'color' => 'success', 'factura' => $factura->id, 'clienteid' => $cliente]);
    }

    // paga una venta al credito
    public function pagarCredito($id)
    {
        $venta = Ventas::find($id);

        $venta->credito = false;

        $venta->save();

        return back()->with(['info' => 'Articulo Cancelado', 'color' => 'success']);
    }

    // imprime la factura de una venta realizada
    public function ventaPrint($idFactura, $idCliente)
    {
        $deuda = DB::table('venta_articulos AS ven')
            ->join('clientes AS cli', 'ven.cliente_id', '=', 'cli.id')
            ->join('articulos AS art', 'ven.articulo_id', '=', 'art.id')
            ->join('facturas AS fac', 'ven.factura_id', '=', 'fac.id')
            ->select(
                'ven.id',
                'ven.factura_id',
                'ven.cantidad',
                'ven.cliente_id',
                'ven.total',

                'ven.descuento',
                'ven.created_at as fechaVenta',
                'cli.nombre',
                'cli.direccion',
                'cli.telefono1',
                'cli.nit',
                'cli.telefono2',
                'art.nombre as nomArt',
                'art.cod_barras',
                'art.descripcion',
                'art.p_venta',
                'fac.descuento as facDescuento',
                'fac.total as facTotal',
                'fac.descripcion as facDesc'
            )
            ->where('ven.factura_id', $idFactura)
            ->get();

        $facturaVenta = Factura::find($idFactura);



        $totalDeuda = 0;
        $nombre = '';
        $fecha = date('Y');
        $nit = 0;
        $descuento_articulo = 0;
        $sin_desc = 0;
        $mayorista = false;
        $facTotal = 0;


        $aux3 = 0;

        foreach ($deuda as $deu) {
            $totalDeuda =  $deu->total + $totalDeuda;
            $nombre = $deu->nombre;
            $fecha = $deu->fechaVenta;
            $nit = $deu->nit;
            $descuento_articulo = $descuento_articulo + $deu->descuento;
            $sin_desc = ($sin_desc + $deu->p_venta) * $deu->cantidad;
            $facTotal = $deu->facTotal;
            $facDesc = $deu->facDesc;
        }




        return view('admin.reports.ventas.venta', ['deuda' => $deuda, 'totalDeuda' => $totalDeuda, 'cliente' => $nombre, 'factura' => $facturaVenta, 'fecha' => $fecha, 'nit' => $nit, 'descuetoArticulos' => $descuento_articulo, 'sin_desc' => $sin_desc, 'mayorista' => $mayorista, 'facTotal' => $facTotal, 'totalSinDecMayo' => $aux3]);
    }


    // devuelve el hoitorial de las ventas realizadas
    public function historial()
    {
        $contado = DB::table('venta_articulos AS ven')
            ->join('clientes AS cli', 'ven.cliente_id', '=', 'cli.id')
            ->join('articulos AS art', 'ven.articulo_id', '=', 'art.id')
            ->select(
                'ven.id',
                'ven.factura_id',
                'ven.articulo_id',
                'ven.cantidad',
                'ven.cliente_id',
                'ven.total',
                'ven.created_at as fechaVenta',
                'cli.nombre',
                'cli.direccion',
                'cli.telefono1',
                'cli.telefono2',
                'art.nombre as nomArt',
                'art.cod_barras',
                'art.descripcion'
            )
            ->groupBy('ven.factura_id')
            ->orderBy('fechaVenta', 'desc')
            ->get();

        // dd($contado);


        return view('admin.ventas.historial', ['contado' => $contado]);
    }

    // elimina una venta y actualiza el stock en los articulos
    public function destroy($id, $idArt)
    {
        $venta = Ventas::where('factura_id', $id)->get();
        foreach ($venta as $venta) {
            $articulo = Articulo::find($venta->articulo_id);
            $articulo->stock = $articulo->stock + $venta->cantidad;
            $articulo->save();
            $venta->delete();
        }
        return back()->with(['info' => 'Venta Anulda Con Exito', 'color' => 'danger']);
    }

    // devuelve la vista para los reportes
    public function reportes()
    {
        return view('admin.ventas.reportes');
    }

    // devuelve la vista para reporte de todas las ventas 
    public function reportesVentasAll(Request $data)
    {
        $ventas = DB::table('venta_articulos AS ven')
            ->join('clientes AS cli', 'ven.cliente_id', '=', 'cli.id')
            ->join('articulos AS art', 'ven.articulo_id', '=', 'art.id')
            ->join('facturas AS fac', 'ven.factura_id', '=', 'fac.id')
            ->select(
                'ven.id',
                'ven.factura_id',
                'ven.articulo_id',
                'ven.cantidad',
                'ven.cliente_id',
                'ven.total',
                'ven.descuento',

                'ven.created_at as fechaVenta',
                'cli.nombre',
                'cli.direccion',
                'cli.telefono1',
                'cli.telefono2',
                'art.nombre as nomArt',
                'art.cod_barras',
                'art.descripcion',
                'art.p_venta',
                'art.p_costo',
                'fac.total as facTotal',
                'fac.descuento  as facDec',
                'fac.id  as facid',
                'fac.descripcion as facDes'
            )
            ->whereMonth('ven.created_at', $data->mes)
            ->whereYear('ven.created_at', $data->ano)

            ->get();

        $ventas_mayoristas = DB::table('venta_articulos AS ven')
            ->join('clientes AS cli', 'ven.cliente_id', '=', 'cli.id')

            ->join('facturas AS fac', 'ven.factura_id', '=', 'fac.id')
            ->select(
                'ven.created_at as fechaVenta',
                'fac.total as facTotal',
                'fac.descuento  as facDec',
                'fac.id  as facid',
                'fac.descripcion as facDes'
            )
            ->whereMonth('ven.created_at', $data->mes)
            ->whereYear('ven.created_at', $data->ano)

            ->groupBy('fac.id')
            ->get();

        $ventas_mayoristas2 = DB::table('venta_articulos AS ven')
            ->join('clientes AS cli', 'ven.cliente_id', '=', 'cli.id')
            ->join('articulos AS art', 'ven.articulo_id', '=', 'art.id')
            ->join('facturas AS fac', 'ven.factura_id', '=', 'fac.id')
            ->select(
                'ven.id',
                'ven.factura_id',
                'ven.articulo_id',
                'ven.cantidad',
                'ven.cliente_id',
                'ven.total',
                'ven.descuento',

                'ven.created_at as fechaVenta',
                'cli.nombre',
                'cli.direccion',
                'cli.telefono1',
                'cli.telefono2',
                'art.nombre as nomArt',
                'art.cod_barras',
                'art.descripcion',
                'art.p_venta',
                'art.p_costo',
                'fac.total as facTotal',
                'fac.descuento  as facDec',
                'fac.id  as facid',
                'fac.descripcion as facDes'
            )
            ->whereMonth('ven.created_at', $data->mes)
            ->whereYear('ven.created_at', $data->ano)

            ->get();
        $ventasEsperadasMayo = 0;

        foreach ($ventas_mayoristas2 as $ventaMayo) {
            $ventasEsperadasMayo = $ventasEsperadasMayo + $ventaMayo->total;
        }

        $total = 0;
        foreach ($ventas as $ven) {
            $total = $ven->total + $total;
            // echo $total, '.', $ven->total, '<br>';
        }

        return view('admin.reports.ventas.ventas', ['ventas' => $ventas, 'total' => $total, 'mes' => $data->mes, 'ano' => $data->ano, 'ventas_mayo' => $ventas_mayoristas, 'total_mayo' => $ventasEsperadasMayo]);
    }

    // devuelve la vista para imprimir reportes de ventas al contado
    public function reportesVentasContado(Request $data)
    {
        $ventas = $this->ventasContado($data->mes, $data->ano);
        $total = 0;
        foreach ($ventas as $ven) {
            $total = $ven->total + $total;
        }
        return view('admin.reports.ventas.ventas', ['ventas' => $ventas, 'total' => $total, 'tipo' => 'al contado', 'mes' => $data->mes, 'ano' => $data->ano]);
    }

    // ---------------------------------------------------
    // ventas all
    public function allventas($mes, $ano)
    {
        $ventas = DB::table('venta_articulos AS ven')
            ->join('clientes AS cli', 'ven.cliente_id', '=', 'cli.id')
            ->join('articulos AS art', 'ven.articulo_id', '=', 'art.id')
            ->join('facturas AS fac', 'ven.factura_id', '=', 'fac.id')
            ->select(
                'ven.id',
                'ven.factura_id',
                'ven.articulo_id',
                'ven.cantidad',
                'ven.cliente_id',
                'ven.total',
                'ven.descuento',

                'ven.created_at as fechaVenta',
                'cli.nombre',
                'cli.direccion',
                'cli.telefono1',
                'cli.telefono2',
                'art.nombre as nomArt',
                'art.cod_barras',
                'art.descripcion',
                'art.p_venta',
                'fac.total as facTotal',
                'fac.descuento  as facDec',
                'fac.id  as facid',
                'fac.descripcion as facDes'
            )
            ->whereMonth('ven.created_at', $mes)
            ->whereYear('ven.created_at', $ano)

            ->get();

        return $ventas;
    }

    // ventas al contado
    public function ventasContado($mes, $ano)
    {
        $contado = DB::table('venta_articulos AS ven')
            ->join('clientes AS cli', 'ven.cliente_id', '=', 'cli.id')
            ->join('articulos AS art', 'ven.articulo_id', '=', 'art.id')
            ->select(
                'ven.id',
                'ven.factura_id',
                'ven.articulo_id',
                'ven.cantidad',
                'ven.cliente_id',
                'ven.total',

                'ven.created_at as fechaVenta',
                'cli.nombre',
                'cli.direccion',
                'cli.telefono1',
                'cli.nit',
                'cli.telefono2',
                'art.nombre as nomArt',
                'art.cod_barras',
                'art.descripcion',
                'art.p_venta',
                'art.p_costo',
                'ven.descuento'
            )
            ->whereMonth('ven.created_at', $mes)
            ->whereYear('ven.created_at', $ano)
            ->get();
        return $contado;
    }


    // retorna los articulos vendidos dell dia actual
    public function ventasDia()
    {
        $ventas = DB::table('venta_articulos AS ven')
            ->join('clientes AS cli', 'ven.cliente_id', '=', 'cli.id')
            ->join('articulos AS art', 'ven.articulo_id', '=', 'art.id')
            ->join('facturas AS fac', 'ven.factura_id', '=', 'fac.id')
            ->select(
                'ven.id',
                'ven.factura_id',
                'ven.articulo_id',
                'ven.cantidad',
                'ven.cliente_id',
                'ven.total',
                'ven.descuento',

                'ven.created_at as fechaVenta',
                'cli.nombre',
                'cli.direccion',
                'cli.telefono1',
                'cli.telefono2',
                'art.nombre as nomArt',
                'art.cod_barras',
                'art.descripcion',
                'art.p_venta',
                'art.p_costo',
                'fac.total as facTotal',
                'fac.descuento  as facDec',
                'fac.id  as facid',
                'fac.descripcion as facDes'
            )
            ->whereDate('ven.created_at', date('Y-m-d'))

            ->get();

        return view('admin.reports.ventas.dia', ['ventas' => $ventas]);
    }

    // ultimos movimientos
    public function movimientos()
    {
        $ventas = DB::table('venta_articulos AS ven')
            ->join('clientes AS cli', 'ven.cliente_id', '=', 'cli.id')
            ->join('articulos AS art', 'ven.articulo_id', '=', 'art.id')
            ->join('facturas AS fac', 'ven.factura_id', '=', 'fac.id')
            ->select(
                'ven.id',
                'ven.factura_id',
                'ven.articulo_id',
                'ven.cantidad',
                'ven.cliente_id',
                'ven.total',
                'ven.descuento',

                'ven.created_at as fechaVenta',
                'cli.nombre',
                'cli.direccion',
                'cli.telefono1',
                'cli.telefono2',
                'art.nombre as nomArt',
                'art.cod_barras',
                'art.descripcion',
                'art.p_venta',
                'art.p_costo',
                'fac.total as facTotal',
                'fac.descuento  as facDec',
                'fac.id  as facid',
                'fac.descripcion as facDes'
            )
            ->whereDate('ven.created_at', date('Y-m-d'))

            ->take(15)
            ->get();


        return view('admin.reports.ventas.dia', ['ventas' => $ventas]);
    }


    // retorna la vista para las estadisticas
    public function estadistica()
    {
        $estadisticasano = $this->estadis();
        $masVendidos = $this->masVendidos();

        $categoria = $this->categoriaEs();


        // dd($categoria);

        return view('admin.reports.ventas.estadistica', ['datos' => $estadisticasano, 'masVendidos' => $masVendidos, 'categoria' => $categoria]);
    }

    // retorna los datos de entradas de efectico
    // para construir las graficas estadisticass
    public function estadis()
    {

        $ventasEnero = $this->ventasMes(1);
        $ventasFebrero = $this->ventasMes(2);
        $ventasMarzo = $this->ventasMes(3);
        $ventasAbril = $this->ventasMes(4);
        $ventasMayo = $this->ventasMes(5);
        $ventasJunio = $this->ventasMes(6);
        $ventasJulio = $this->ventasMes(7);
        $ventasAgosto = $this->ventasMes(8);
        $ventasSptiembre = $this->ventasMes(9);
        $ventasOctubre = $this->ventasMes(10);
        $ventasNoviembre = $this->ventasMes(11);
        $ventasDiciembre = $this->ventasMes(12);
        $totales = array();
        $total1 = 0;
        $total2 = 0;
        $total3 = 0;
        $total4 = 0;
        $total5 = 0;
        $total6 = 0;
        $total7 = 0;
        $total8 = 0;
        $total9 = 0;
        $total10 = 0;
        $total11 = 0;
        $total12 = 0;

        foreach ($ventasEnero as $venta) {
            $total1 = $total1 +  $venta->cuadre;
        }
        foreach ($ventasFebrero as $venta) {
            $total2 = $total2 +  $venta->cuadre;
        }
        foreach ($ventasMarzo as $venta) {
            $total3 = $total3 +  $venta->cuadre;
        }

        foreach ($ventasAbril as $venta) {
            $total4 = $total4 +  $venta->cuadre;
        }
        foreach ($ventasMayo as $venta) {
            $total5 = $total5 +  $venta->cuadre;
        }
        foreach ($ventasJunio as $venta) {
            $total6 = $total6 +  $venta->cuadre;
        }
        foreach ($ventasJulio as $venta) {
            $total7 = $total7 +  $venta->cuadre;
        }
        foreach ($ventasAgosto as $venta) {
            $total8 = $total8 +  $venta->cuadre;
        }
        foreach ($ventasSptiembre as $venta) {
            $total9 = $total9 +  $venta->cuadre;
        }
        foreach ($ventasOctubre as $venta) {
            $total10 = $total10 +  $venta->cuadre;
        }
        foreach ($ventasNoviembre as $venta) {
            $total11 = $total11 +  $venta->cuadre;
        }
        foreach ($ventasDiciembre as $venta) {
            $total12 = $total12 +  $venta->cuadre;
        }


        array_push($totales, $total1, $total2, $total3, $total4, $total5, $total6, $total7, $total8, $total9, $total10, $total11, $total12);


        return $totales;
    }

    // retorna  los cuadres de caja por mes
    public function ventasMes($mes)
    {
        $data =
            DB::table('cuadre')
            ->whereMonth('created_at', $mes)
            ->whereYear('created_at', date('Y'))
            ->get();
        return $data;
    }


    // retorna un listadode articulos mas vendidos
    public function masVendidos()
    {
        $data =  DB::table('venta_articulos')
            ->join('articulos as art', 'venta_articulos.articulo_id', '=', 'art.id')
            ->select(DB::raw('COUNT(venta_articulos.articulo_id) as idArticulo'), 'venta_articulos.factura_id', 'art.nombre', 'art.descripcion')
            ->groupBy('venta_articulos.articulo_id')
            ->orderBy(DB::raw('venta_articulos.articulo_id'), 'desc')
            ->limit(15)
            ->get();
        $data2 = array();
        foreach ($data as $d) {
            array_push($data2, $d);
        }
        return $data2;
    }

    // retorna un listado de los clientes mas grecuentes
    public function clientesPremium()
    {
        // cliente_id

        return  DB::table('venta_articulos')
            ->join('clientes', 'venta_articulos.cliente_id', '=', 'clientes.id')
            ->select(DB::raw('COUNT(venta_articulos.cliente_id) as idcliente'), 'venta_articulos.factura_id', 'clientes.nombre')
            ->where('cliente_id', '!=', 14)
            ->groupBy('venta_articulos.cliente_id')
            ->orderBy(DB::raw('venta_articulos.cliente_id'), 'desc')
            ->limit(10)
            ->get();
    }

    public function movimientoMes()
    {
        $dias = array();
        for ($i = 0; $i < 32; $i++) {
            $data =
                DB::table('cuadre')
                ->whereDay('created_at', $i)
                ->whereMonth('created_at', date('m'))
                ->whereYear('created_at', date('Y'))
                ->get();
            array_push($dias, $data);
        }
        return $dias;
    }

    // retorna las ventas filtradas por fecha de forma simple
    public function fucturasSimples(Request $request)
    {
        $fac = DB::table('facturas as fac')
            ->select('fac.*')
            ->whereBetween('created_at', [$request->desde, $request->hasta])
            ->get();
        return view('admin.reports.ventas.facturassimples', ['data' => $fac]);
    }

    // RETORNA LOS ARTICULOS QUE ESTE MAS FRECUENTES EN UN CATEGORIA
    public function categoriaEs()
    {
        return  DB::table('articulos as art')
            ->join('categoria as cat', 'art.categoria_id', '=', 'cat.id')
            ->select('cat.nombre', DB::raw('Sum(art.categoria_id) as frecu'))
            ->groupBy('art.categoria_id')
            ->get();
    }

    // retorna la respuesta en json para peticion ajax
    public function findArticle(Request $request)
    {
        $articulo = Articulo::orwhere('descripcion', 'like', '%' . $request->codigo . '%')
            ->orwhere('cod_barras', 'like', '%' . $request->codigo . '%')
            ->orwhere('nombre', 'like', '%' . $request->codigo . '%')
            ->get();

        return Response::json($articulo);
    }

    // retorna las ventas por tipo de articulo
    public function reportToTipoArticle(Request $request)
    {

        // dd($request->inicio, $request->fin);
        $ventas = DB::table('venta_articulos AS ven')
            ->join('clientes AS cli', 'ven.cliente_id', '=', 'cli.id')
            ->join('articulos AS art', 'ven.articulo_id', '=', 'art.id')
            ->join('facturas AS fac', 'ven.factura_id', '=', 'fac.id')
            ->select(
                'ven.id',
                'ven.factura_id',
                'ven.articulo_id',
                'ven.cantidad',
                'ven.cliente_id',
                'ven.total',
                'ven.descuento',
                'ven.created_at',
                'cli.nombre',
                'cli.direccion',
                'cli.telefono1',
                'cli.telefono2',
                'art.nombre as nomArt',
                'art.cod_barras',
                'art.descripcion',
                'art.p_venta',
                'art.p_costo',
                'art.tipo',
                'fac.total as facTotal',
                'fac.descuento  as facDec',
                'fac.id  as facid',
                'fac.descripcion as facDes'
            )
            ->whereBetween('ven.created_at', [$request->inicio, $request->fin])
            ->where('art.tipo', $request->tipo)
            ->get();

            // dd($ventas);

        $total = 0;
        foreach ($ventas as $ven) {
            $total = $ven->total + $total;
        }


        return view('admin.reports.ventas.tipo', ['ventas' => $ventas, 'total' => $total]);
    }
}
